@@ -30,32 +30,28 @@ trap 'catch $? $LINENO' ERR TERM INT
 # Lastly, we take give the component CSVs to the csv-merger that combines all
 # of the manifests into a single, unified, ClusterServiceVersion.
 
-function get_image_digest() {
-  if [[ ! -f ${PROJECT_ROOT}/tools/digester/digester ]]; then
-    (
-      cd "${PROJECT_ROOT}/tools/digester"
-      go build .
-    )
-  fi
-
-  local image
-  image=$("${PROJECT_ROOT}/tools/digester/digester" -image "$1" "$2")
-  echo "${image}"
-}
-
 PROJECT_ROOT="$(readlink -e $(dirname "${BASH_SOURCE[0]}")/../)"
 source "${PROJECT_ROOT}"/hack/config
+# update image digests
+"${PROJECT_ROOT}"/automation/digester/update_images.sh
 source "${PROJECT_ROOT}"/deploy/images.env
 
+HCO_OPERATOR_IMAGE=${HCO_OPERATOR_IMAGE:-quay.io/kubevirt/hyperconverged-cluster-operator:${CSV_VERSION}-unstable}
+HCO_WEBHOOK_IMAGE=${HCO_WEBHOOK_IMAGE:-quay.io/kubevirt/hyperconverged-cluster-webhook:${CSV_VERSION}-unstable}
+DIGEST_LIST="${DIGEST_LIST},${HCO_OPERATOR_IMAGE}|hyperconverged-cluster-operator,${HCO_WEBHOOK_IMAGE}|hyperconverged-cluster-webhook"
+
 DEPLOY_DIR="${PROJECT_ROOT}/deploy"
 CRD_DIR="${DEPLOY_DIR}/crds"
 OLM_DIR="${DEPLOY_DIR}/olm-catalog"
+CSV_VERSION=${CSV_VERSION}
+CSV_TIMESTAMP=$(date +%Y%m%d%H%M -u)
 PACKAGE_NAME="community-kubevirt-hyperconverged"
 CSV_DIR="${OLM_DIR}/${PACKAGE_NAME}/${CSV_VERSION}"
 DEFAULT_CSV_GENERATOR="/usr/bin/csv-generator"
 SSP_CSV_GENERATOR="/csv-generator"
 
 INDEX_IMAGE_DIR=${DEPLOY_DIR}/index-image
+CSV_INDEX_IMAGE_DIR="${INDEX_IMAGE_DIR}/${PACKAGE_NAME}/${CSV_VERSION}"
 
 OPERATOR_NAME="${OPERATOR_NAME:-kubevirt-hyperconverged-operator}"
 OPERATOR_NAMESPACE="${OPERATOR_NAMESPACE:-kubevirt-hyperconverged}"
@@ -92,7 +88,7 @@ function gen_csv() {
     --prefix="${operatorName}" \
     --suffix-format="%02d.${CRD_EXT}" \
     $crds \
-    "/---/" "{*}"
+    "/^---$/" "{*}"
 }
 
 function create_virt_csv() {
@@ -246,21 +242,14 @@ hppCsv="${TEMPDIR}/${hhpFile}.${CSV_EXT}"
 vmImportFile=$(create_vm_import_csv)
 importCsv="${TEMPDIR}/${vmImportFile}.${CSV_EXT}"
 csvOverrides="${TEMPDIR}/csv_overrides.${CSV_EXT}"
-
+keywords="  keywords:
+  - KubeVirt
+  - Virtualization
+  - VM"
 cat > ${csvOverrides} <<- EOM
 ---
 spec:
-  links:
-  - name: KubeVirt project
-    url: https://kubevirt.io
-  - name: Source Code
-    url: https://github.com/kubevirt/hyperconverged-cluster-operator
-  maintainers:
-  - email: kubevirt-dev@googlegroups.com
-    name: KubeVirt project
-  maturity: alpha
-  provider:
-    name: KubeVirt project
+$keywords
 EOM
 
 # Write HCO CRDs
@@ -278,6 +267,7 @@ popd
 rm -fr "${CSV_DIR}"
 mkdir -p "${CSV_DIR}/metadata" "${CSV_DIR}/manifests"
 
+
 cat << EOF > "${CSV_DIR}/metadata/annotations.yaml"
 annotations:
   operators.operatorframework.io.bundle.channel.default.v1: ${CSV_VERSION}
@@ -301,26 +291,6 @@ for csv in "${csvs[@]}"; do
   grep -E "^ *image: [a-zA-Z0-9/\.:@\-]+$" ${csv}
 done
 
-if [[ -n ${OPERATOR_IMAGE} ]]; then
-  TEMP_IMAGE_NAME=$(get_image_digest "${OPERATOR_IMAGE}")
-  DIGEST_LIST="${DIGEST_LIST/${HCO_OPERATOR_IMAGE}/${TEMP_IMAGE_NAME}}"
-  HCO_OPERATOR_IMAGE=${TEMP_IMAGE_NAME}
-fi
-
-if [[ -n ${WEBHOOK_IMAGE} ]]; then
-  TEMP_IMAGE_NAME=$(get_image_digest "${WEBHOOK_IMAGE}")
-  if [[ -n ${HCO_WEBHOOK_IMAGE} ]]; then
-    DIGEST_LIST="${DIGEST_LIST/${HCO_WEBHOOK_IMAGE}/${TEMP_IMAGE_NAME}}"
-  else
-    DIGEST_LIST="${DIGEST_LIST},${TEMP_IMAGE_NAME}"
-  fi
-  HCO_WEBHOOK_IMAGE=${TEMP_IMAGE_NAME}
-fi
-
-if [[ -z ${HCO_WEBHOOK_IMAGE} ]]; then
-  HCO_WEBHOOK_IMAGE="${HCO_OPERATOR_IMAGE}"
-fi
-
 # Build and write deploy dir
 (cd ${PROJECT_ROOT}/tools/manifest-templator/ && go build)
 ${PROJECT_ROOT}/tools/manifest-templator/manifest-templator \
@@ -334,6 +304,7 @@ ${PROJECT_ROOT}/tools/manifest-templator/manifest-templator \
   --vmimport-csv="$(<${importCsv})" \
   --ims-conversion-image-name="${CONVERSION_IMAGE}" \
   --ims-vmware-image-name="${VMWARE_IMAGE}" \
+  --kv-virtiowin-image-name="${KUBEVIRT_VIRTIO_IMAGE}" \
   --operator-namespace="${OPERATOR_NAMESPACE}" \
   --smbios="${SMBIOS}" \
   --hco-kv-io-version="${CSV_VERSION}" \
@@ -348,6 +319,14 @@ ${PROJECT_ROOT}/tools/manifest-templator/manifest-templator \
   --webhook-image="${HCO_WEBHOOK_IMAGE}"
 (cd ${PROJECT_ROOT}/tools/manifest-templator/ && go clean)
 
+if [[ "$1" == "UNIQUE"  ]]; then
+  CSV_VERSION_PARAM=${CSV_VERSION}-${CSV_TIMESTAMP}
+  ENABLE_UNIQUE="true"
+else
+  CSV_VERSION_PARAM=${CSV_VERSION}
+  ENABLE_UNIQUE="false"
+fi
+
 # Build and merge CSVs
 CSV_DIR=${CSV_DIR}/manifests
 ${PROJECT_ROOT}/tools/csv-merger/csv-merger \
@@ -360,14 +339,17 @@ ${PROJECT_ROOT}/tools/csv-merger/csv-merger \
   --vmimport-csv="$(<${importCsv})" \
   --ims-conversion-image-name="${CONVERSION_IMAGE}" \
   --ims-vmware-image-name="${VMWARE_IMAGE}" \
-  --csv-version=${CSV_VERSION} \
+  --kv-virtiowin-image-name="${KUBEVIRT_VIRTIO_IMAGE}" \
+  --csv-version=${CSV_VERSION_PARAM} \
   --replaces-csv-version=${REPLACES_CSV_VERSION} \
   --hco-kv-io-version="${CSV_VERSION}" \
   --spec-displayname="KubeVirt HyperConverged Cluster Operator" \
   --spec-description="$(<${PROJECT_ROOT}/docs/operator_description.md)" \
+  --metadata-description="A unified operator deploying and controlling KubeVirt and its supporting operators with opinionated defaults" \
   --crd-display="HyperConverged Cluster Operator" \
   --smbios="${SMBIOS}" \
   --csv-overrides="$(<${csvOverrides})" \
+  --enable-unique-version=${ENABLE_UNIQUE} \
   --kubevirt-version="${KUBEVIRT_VERSION}" \
   --cdi-version="${CDI_VERSION}" \
   --cnao-version="${NETWORK_ADDONS_VERSION}" \
@@ -379,6 +361,11 @@ ${PROJECT_ROOT}/tools/csv-merger/csv-merger \
   --operator-image-name="${HCO_OPERATOR_IMAGE}" \
   --webhook-image-name="${HCO_WEBHOOK_IMAGE}" > "${CSV_DIR}/${OPERATOR_NAME}.v${CSV_VERSION}.${CSV_EXT}"
 
+rendered_csv="$(cat "${CSV_DIR}/${OPERATOR_NAME}.v${CSV_VERSION}.${CSV_EXT}")"
+rendered_keywords="$(echo "$rendered_csv" |grep 'keywords' -A 3)"
+# assert that --csv-overrides work
+[ "$keywords" == "$rendered_keywords" ]
+
 # Copy all CRDs into the CRD and CSV directories
 rm -f ${CRD_DIR}/*
 cp -f ${TEMPDIR}/*.${CRD_EXT} ${CRD_DIR}
@@ -392,13 +379,28 @@ cp -f ${TEMPDIR}/*.${CRD_EXT} ${CSV_DIR}
 ${PROJECT_ROOT}/tools/csv-merger/csv-merger --crds-dir=${CRD_DIR}
 (cd ${PROJECT_ROOT}/tools/csv-merger/ && go clean)
 
+if [[ "$1" == "UNIQUE"  ]]; then
+  # Add the current CSV_TIMESTAMP to the currentCSV in the packages file
+  sed -Ei "s/(currentCSV: ${OPERATOR_NAME}.v${CSV_VERSION}).*/\1-${CSV_TIMESTAMP}/" \
+   ${PACKAGE_DIR}/kubevirt-hyperconverged.package.yaml
+fi
+
 # Intentionally removing last so failure leaves around the templates
 rm -rf ${TEMPDIR}
 
+# If the only change in the CSV file is its "created_at" field, rollback this change as it causes git conflicts for
+# no good reason.
+CSV_FILE="${CSV_DIR}/kubevirt-hyperconverged-operator.v${CSV_VERSION}.${CSV_EXT}"
+if git difftool -y --trust-exit-code --extcmd=./hack/diff-csv.sh ${CSV_FILE}; then
+  git checkout ${CSV_FILE}
+fi
+
+# Prepare files for index-image files that will be used for testing in openshift CI
 rm -rf "${INDEX_IMAGE_DIR:?}"
 mkdir -p "${INDEX_IMAGE_DIR:?}/${PACKAGE_NAME}"
 cp -r "${CSV_DIR%/*}" "${INDEX_IMAGE_DIR:?}/${PACKAGE_NAME}/"
 cp "${OLM_DIR}/bundle.Dockerfile" "${INDEX_IMAGE_DIR:?}/"
+cp "${OLM_DIR}/Dockerfile.bundle.ci-index-image-upgrade" "${INDEX_IMAGE_DIR:?}/"
 
 INDEX_IMAGE_CSV="${INDEX_IMAGE_DIR}/${PACKAGE_NAME}/${CSV_VERSION}/manifests/kubevirt-hyperconverged-operator.v${CSV_VERSION}.${CSV_EXT}"
 sed -r -i "s|createdAt: \".*\$|createdAt: \"2020-10-23 08:58:25\"|; s|quay.io/kubevirt/hyperconverged-cluster-operator.*$|+IMAGE_TO_REPLACE+|; s|quay.io/kubevirt/hyperconverged-cluster-webhook.*$|+WEBHOOK_IMAGE_TO_REPLACE+|" ${INDEX_IMAGE_CSV}
